#include "jpeg.h"

bool jpeg::read(const char * filename) {
	FILE * fp;
	if ((fp = fopen(filename, "rb")) == NULL) {
		return false;
	}
	read(fp);
	fclose(fp);
	return true;
}

void jpeg::read(FILE * fp) {
	if(coef_arrays != NULL)
		jpeg_abort_decompress(&srcinfo);

	jpeg_stdio_src(&srcinfo, fp);

	jpeg_save_markers(&srcinfo, JPEG_COM, 0xFFFF);
	for (int i = 0; i < 16; ++i)
		jpeg_save_markers(&srcinfo, JPEG_APP0 + i, 0xFFFF);
	
	jpeg_read_header(&srcinfo, TRUE);
	coef_arrays = jpeg_read_coefficients(&srcinfo);
}

bool jpeg::write(const char * filename) {
	FILE * fp;
	if ((fp = fopen(filename, "wb")) == NULL) {
		return false;
	}
	write(fp);
	fclose(fp);
	return true;
}

void jpeg::write(FILE * fp) {
	jpeg_stdio_dest(&dstinfo, fp);

	jpeg_copy_critical_parameters(&srcinfo, &dstinfo);

	jpeg_write_coefficients(&dstinfo, coef_arrays);
	
	jpeg_saved_marker_ptr marker;
	for (marker = srcinfo.marker_list; marker != NULL; marker = marker->next) {
		if (dstinfo.write_JFIF_header &&
	marker->marker == JPEG_APP0 &&
	marker->data_length >= 5 &&
	GETJOCTET(marker->data[0]) == 0x4A &&
	GETJOCTET(marker->data[1]) == 0x46 &&
	GETJOCTET(marker->data[2]) == 0x49 &&
	GETJOCTET(marker->data[3]) == 0x46 &&
	GETJOCTET(marker->data[4]) == 0)
			continue;			/* reject duplicate JFIF */
		if (dstinfo.write_Adobe_marker &&
	marker->marker == JPEG_APP0+14 &&
	marker->data_length >= 5 &&
	GETJOCTET(marker->data[0]) == 0x41 &&
	GETJOCTET(marker->data[1]) == 0x64 &&
	GETJOCTET(marker->data[2]) == 0x6F &&
	GETJOCTET(marker->data[3]) == 0x62 &&
	GETJOCTET(marker->data[4]) == 0x65)
			continue;			/* reject duplicate Adobe */
		jpeg_write_marker(&dstinfo, marker->marker, marker->data, marker->data_length);
	}
	
	jpeg_finish_compress(&dstinfo);
}

const char * jpeg::jpeg_color_space_str() const {
	switch (srcinfo.jpeg_color_space) {
		case JCS_GRAYSCALE: return "monochrome";
		case JCS_RGB: return "red/green/blue as specified by the RGB_RED, RGB_GREEN, RGB_BLUE, and RGB_PIXELSIZE macros";
		case JCS_YCbCr: return "Y/Cb/Cr (also known as YUV)";
		case JCS_CMYK: return "C/M/Y/K";
		case JCS_YCCK: return "Y/Cb/Cr/K";
		case JCS_EXT_RGB: return "red/green/blue";
		case JCS_EXT_RGBX: return "red/green/blue/x";
		case JCS_EXT_BGR: return "blue/green/red";
		case JCS_EXT_BGRX: return "blue/green/red/x";
		case JCS_EXT_XBGR: return "x/blue/green/red";
		case JCS_EXT_XRGB: return "x/red/green/blue";
		case JCS_EXT_RGBA: return "red/green/blue/alpha";
		case JCS_EXT_BGRA: return "blue/green/red/alpha";
		case JCS_EXT_ABGR: return "alpha/blue/green/red";
		case JCS_EXT_ARGB: return "alpha/red/green/blue";
		default: return "error/unspecified";
	}
}
