#ifndef JPEG_H_INCLUDED
#define JPEG_H_INCLUDED

#include <cstdio>
#include "jpeglib.h"

class jpeg {
	jpeg_decompress_struct srcinfo;
	jpeg_compress_struct dstinfo;
	jpeg_error_mgr jsrcerr;
	jpeg_error_mgr jdsterr;
	jvirt_barray_ptr * coef_arrays;
public:
	jpeg() : coef_arrays(NULL) {
		srcinfo.err = jpeg_std_error(&jsrcerr);
		dstinfo.err = jpeg_std_error(&jdsterr);
		jpeg_create_decompress(&srcinfo);
		jpeg_create_compress(&dstinfo);
	}

	~jpeg() {
		jpeg_destroy_decompress(&srcinfo);
		jpeg_destroy_compress(&dstinfo);	
	}

	bool read(const char * filename);

	void read(FILE * fp);

	bool write(const char * filename);

	void write(FILE * fp);

	JBLOCKARRAY dct_coef(int component_index) {
		return (srcinfo.mem->access_virt_barray)((j_common_ptr)&srcinfo, coef_arrays[component_index], 0, srcinfo.comp_info[component_index].v_samp_factor, TRUE);
	}

	JDIMENSION image_width() const {
		return srcinfo.image_width;
	}

	JDIMENSION image_height() const {
		return srcinfo.image_height;
	}

	J_COLOR_SPACE jpeg_color_space() const {
		return srcinfo.jpeg_color_space;
	}

	const char * jpeg_color_space_str() const;

	int num_components() const {
		return srcinfo.num_components;
	}

	JDIMENSION output_width() const {
		return srcinfo.output_width;
	}

	JDIMENSION output_height() const {
		return srcinfo.output_height;
	}

	int component_id(int component_index) const {
		return srcinfo.comp_info[component_index].component_id;
	}

	JDIMENSION width_in_blocks(int component_index) const {
		return srcinfo.comp_info[component_index].width_in_blocks;
	}

	JDIMENSION height_in_blocks(int component_index) const {
		return srcinfo.comp_info[component_index].height_in_blocks;
	}

	int h_samp_factor(int component_index) const {
		return srcinfo.comp_info[component_index].h_samp_factor;
	}

	int v_samp_factor(int component_index) const {
		return srcinfo.comp_info[component_index].v_samp_factor;
	}
	
	UINT16 * quant_table(int component_index) const {
		// Returns pointer to quantization table of size DCTSIZE2 in natural array order.
		return srcinfo.comp_info[component_index].quant_table->quantval;
	}
};

#endif
