#include "jpeg.h"
#include <iostream>
#include <cstdlib>

using namespace std;

int main(int argc, char * argv[])
{
	if (argc < 2) {
		cerr << "Usage: jpeginfo file [component_index]\n";
		cerr << "This program displays some basic information about the given JPEG file.\n";
		cerr << "If component_index is specified, then the program displays the quantised DCT\n";
		cerr << "coefficients of the component in natural array order.\n";
		return 1;
	}

	jpeg im;

	if (!im.read(argv[1])) {
		cerr << argv[0] << ": Failed to open " << argv[1] << '\n';
		return 1;
	}

	if (argc == 2) {
		cout << "Image width: " << im.image_width() << '\n';
		cout << "Image height: " << im.image_height() << '\n';
		cout << "Number of components: " << im.num_components() << '\n';
		cout << "Color space: " << im.jpeg_color_space_str() << '\n';
		cout << "Scaled image width: " << im.output_width() << '\n';
		cout << "Scaled image height: " << im.output_height() << '\n';

		for (int ci = 0; ci < im.num_components(); ++ci) {
			cout << '\n';
			cout << "Component " << ci << '\n';
			cout << "  Width in blocks: " << im.width_in_blocks(ci) << '\n';
			cout << "  Height in blocks: " << im.height_in_blocks(ci) << '\n';
			cout << "  Horizontal sampling factor: " << im.h_samp_factor(ci) << '\n';
			cout << "  Vertical sampling factor: " << im.v_samp_factor(ci) << '\n';
			cout << "  Quantization table:\n";

			for (int i = 0; i < DCTSIZE; ++i) {
				cout << "  ";
				for (int j = 0; j < DCTSIZE; ++j) {
					cout.width(4);
					cout << im.quant_table(ci)[i*DCTSIZE + j];
				}
				cout << '\n';
			}
		}
	} else {
		int ci = atoi(argv[2]);
		if (ci < 0 || ci >= im.num_components()) {
			cerr << argv[0] << ": Invalid component index.\n";
			return 1;
		}

		JBLOCKARRAY buf = im.dct_coef(ci);
		cout << '{';
		for (JDIMENSION row = 0; row < im.height_in_blocks(ci); ++row) {
			if(row != 0)
				cout << ",\n";
			cout << '{';
			for (JDIMENSION col = 0; col < im.width_in_blocks(ci); ++col) {
				if(col != 0)
					cout << ",\n";
				cout << '{' << buf[row][col][0];
				for (JDIMENSION i = 1; i < DCTSIZE2; ++i)
					cout << ',' << buf[row][col][i];
				cout << '}';
			}
			cout << '}';
		}
		cout << "}\n";
	}

	return 0;
}
